package com.gree.mall.manager.controller.material.manage;

import com.gree.mall.manager.constant.Constant;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.helper.ResponseHelper;
import com.gree.mall.manager.logic.material.manage.WorkerOldRefundManageLogic;
import com.gree.mall.manager.utils.excel.ExcelUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.InputStream;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;

@Slf4j
@RestController
@Api(value = "师傅旧件返还管理", tags ={"师傅旧件返还管理"} )
@RequestMapping(value = "/worker/old-refund", produces = "application/json; charset=utf-8")
public class WorkerOldRefundController {

    @Resource
    WorkerOldRefundManageLogic workerOldRefundManageLogic;
    @Resource
    RedisLockRegistry redisLockRegistry;

    @ApiOperation("旧件返还结算状态导入")
    @PostMapping("/import-settlement-state")
    public ResponseHelper importSettlementState(
            @ApiParam(required = true, value = "附件") @RequestPart("file") MultipartFile file,
            @ApiParam(required = true, value = "0-旧件返还 1-保外新件转销售") @RequestParam Integer flag
    ) throws Exception {
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.NEW_REFUND + "import");
        if (!obtain.tryLock(10, TimeUnit.SECONDS)) {
            throw new RemoteServiceException("获取导入结算状态锁超时");
        }
        try {
            String s;
            if (flag.compareTo(0) == 0) {
                InputStream inputStream = file.getInputStream();
                s = workerOldRefundManageLogic.importInnerSettlement(inputStream);
            } else {
                List<Object> objects = ExcelUtils.importExcel(file);
                s = workerOldRefundManageLogic.importOutsideSettlement(objects);
            }
            return ResponseHelper.success(s);
        } finally {
            obtain.unlock();
        }
    }
}
